
//*****************************************************************************
//
// File Name	: 'global.h'
// Title			: AVR project global include 
// Author		: Pascal Stang
// Created		: 7/12/2001
// Revised		: 4/30/2002
// Version		: 1.1
// Target MCU	: Atmel AVR series
// Editor Tabs	: 3
//
//	Description : This include file is designed to contain items useful to all
//					code files and projects.
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************


#ifndef GLOBAL_H
#define GLOBAL_H

// Code compatibility to new AVR-libc
// outb(), inb(), BV(), sbi(), cbi(), sei(), cli()
#ifndef outb
	#define	outb(addr, data)	addr = (data)
#endif
#ifndef inb
	#define	inb(addr)			(addr)
#endif

#ifndef outp
	#define	outp(data, addr)	addr = (data)
#endif

#ifndef inp
	#define	inp(addr)			(addr)
#endif

#ifndef BV
	#define BV(bit)			(1<<(bit))
#endif
#ifndef cbi
	#define cbi(reg,bit)	reg &= ~(BV(bit))
#endif
#ifndef sbi
	#define sbi(reg,bit)	reg |= (BV(bit))
#endif
#ifndef cli
	#define cli()			__asm__ __volatile__ ("cli" ::)
#endif
#ifndef sei
	#define sei()			__asm__ __volatile__ ("sei" ::)
#endif

// port address helpers
#define DDR(x) ((x)-1)    // address of data direction register of port x
#define PIN(x) ((x)-2)    // address of input register of port x

// min/max macros
#define MIN(a,b)			((a<b)?(a):(b))
#define MAX(a,b)			((a>b)?(a):(b))
#define ABS(x)				(((x)>0)?(x):(-(x)))

// macro for reading 16bit words from program memory
#define PRG_RDW(a)		( (PRG_RDB((unsigned char*)(a)) & 0x00FF) | ((PRG_RDB((unsigned char*)(a)+1))<<8) )

// constants
#define PI		3.14159265359

#ifndef WIN32
	// true/false defines
	#define FALSE	0
	#define TRUE	-1
#endif

// datatype definitions macros
typedef unsigned char  u08;
typedef          char  s08;
typedef unsigned short u16;
typedef          short s16;
typedef unsigned long  u32;
typedef          long  s32;
typedef unsigned long long u64;
typedef          long long s64;

// maximum value that can be held
// by unsigned data types (8,16,32bits)
#define MAX_U08	255
#define MAX_U16	65535
#define MAX_U32	4294967296

// maximum values that can be held
// by signed data types (8,16,32bits)
#define MIN_S08	-128
#define MAX_S08	127
#define MIN_S16	-32768
#define MAX_S16	32767
#define MIN_S32	-2147483648
#define MAX_S32	2147483647

#ifndef WIN32
	// more type redefinitions
	typedef unsigned char   BOOL;
	typedef unsigned char	BYTE;
	typedef unsigned int	WORD;
	typedef unsigned long	DWORD;

	typedef unsigned char	UCHAR;
	typedef unsigned int	UINT;
	typedef unsigned short  USHORT;
	typedef unsigned long	ULONG;

	typedef char  CHAR;
	typedef int   INT;
	typedef long  LONG;
#endif

// system dependent defines

// CPU clock speed 
//#define F_CPU        1000000               		// 1MHz processor
#define F_CPU        4000000               		// 4MHz processor
//#define F_CPU        7372800               		// 7.37MHz processor
//#define F_CPU        4000000               		// 4MHz processor
//#define F_CPU        3686400               		// 3.69MHz processor
#define CYCLES_PER_US ((F_CPU+500000)/1000000) 	// cpu cycles per microsecond

// address space defines
// internal ram
#define INT_RAM_BASE					0x0060
#define INT_RAM_END					0x045f

//############################

#define MAXKONST	26
#define KONSTADDR	2 // Adresse der ersten Konstanten im eeprom

//*********************
// Sensor Makros

#define TASTE_L		bit_is_clear(PINB,4)
#define TASTE_M		bit_is_clear(PIND,3)
#define TASTE_R		bit_is_clear(PIND,4)
#define TASTE		(TASTE_L||TASTE_M||TASTE_R)

#define BAT_SIG		(bit_is_set(PIND,2)) // pind 2 ist auch interrupt0 eingang
#define BAT_SET_OUT	(sbi(DDRD,2))
#define BAT_SET_IN	(cbi(DDRD,2))
#define BAT_SIG_ON	(sbi(PORTD,2))
#define BAT_SIG_OFF	(cbi(PORTD,2))
 
//Liniensensoren
#define LEDS_ON		(sbi(PORTB,5))
#define LEDS_OFF	(cbi(PORTB,5))
// von links nach rechts
#define LINIE1		(AD(3))
#define LINIE2		(AD(2))
#define LINIE3		(AD(1))
#define LINIE4		(AD(0))
#define LINIE5		(AD(7))
#define LINIE6		(AD(6))
//----------------


//Was hngt an welchem ADC-Eingang
//----------------


//zur Motorsteuerung
#define SET_MOTOR_L(x)		(PWMA_Set(x))
#define SET_MOTOR_R(x)		(PWMB_Set(x))
//----------------





#define mem(addr) 		( *(char*)(addr) )

#define setbit(value,bit)   	( value =value|BV(bit) )
#define clearbit(value,bit) 	( value &= ~BV(bit) )
#define bitset(value,bit)   	( value & (1<<(bit)) ) // bitset?

#define TIME_IS_OVER(time, numofms) (GetTime()>(time)+(numofms))

#define EVER ;;

typedef unsigned long t_Time;

extern s16 konst[MAXKONST];
extern void ReadKonst(u08 aufgabe);
extern void SaveKonst(u08 aufgabe);


#endif
